#!/usr/bin/perl

use Getopt::Long;
use strict;

my ( $opt_port, $opt_mysqltest, $opt_mysqladmin, $opt_start_server, $opt_duration, $opt_help ) = 
( 10000, '../client/mysqltest', '../client/mysqladmin', 1, 300, undef );

GetOptions( "port=i" => \$opt_port,
			"mysqltest=s" => \$opt_mysqltest,
			"mysqladmin=s" => \$opt_mysqladmin,
			"start-server=i" => \$opt_start_server, 
			"duration=i" => \$opt_duration,
			"help" => \$opt_help 
			) || usage();
           
if ( $opt_help ) {
	usage();
	exit;
}

unless ( -e "./mysql-test-run.pl" ) {
	die "ERROR: ./mysql-test.run.pl does not exist\n";
}

unless ( -e "./mysql-stress-test.pl" ) {
	die "ERROR: ./mysql-stress-test.pl does not exist\n";
}
           
unless ( -e $opt_mysqltest ) {
	usage();
	die "ERROR: Could not find mysqltest\n";
}

unless ( -e $opt_mysqladmin ) {
	usage();
	die "ERROR: Could not find mysqladmin\n";
}


unless ( $opt_port > 0 ) {
	usage();
	die "ERROR: server port has wrong value: $opt_port\n";
}
           
           


# parent will run mysql-stress-test
# one child will ping server 
# another child will run the server (if needed)

my $stress_runner_pid = $$;
my $server_runner_pid = undef;
    
if ( $opt_start_server ) {
	my $server_runner_pid = fork();
	die "Could not fork for server runner\n" unless defined $server_runner_pid;
	
	if ( $server_runner_pid == 0 ) {
		my $mtr_build_thread = ($opt_port - 10000)/10;
		system( "perl ./mysql-test-run.pl --mysqld=--innodb --mtr-build-thread=$mtr_build_thread --start alias" );
		print "Server is not running anymore...\n";
		exit;
	}
}

my $server_found = 0;
until ( $server_found ) {
	sleep 5;
	print "Waiting for server to start...\n";
	system( "$opt_mysqladmin -uroot --port=$opt_port --protocol=tcp --silent ping");
	$server_found = ( $? == 0 );
} 

my $watchdog_pid = fork();
die "Could not fork for watchdog\n" unless defined $watchdog_pid;

if ( $watchdog_pid ) # parent 
{ 
	mkdir "./crash/res_run" unless ( -d "./crash/res_run" );
	mkdir "./crash/log_run" unless ( -d "./crash/log_run" );
	
	system( "perl ./mysql-stress-test.pl --server-host=127.0.0.1 --server-port=$opt_port --stress-suite-basedir=./crash "
		. " --mysqltest=$opt_mysqltest --stress-basedir=./crash/res_run --stress-tests-file=./crash/crash.txt "
		. " --server-logs-dir=./crash/log_run --threads=2 --test-duration=$opt_duration --server-user=root "
		. " --log-error-details --verbose --cleanup --stress-init-file=./crash/init_crash.txt > ./crash/test.out" );
	if ( defined $server_runner_pid ) {
		system( "$opt_mysqladmin -uroot --port=$opt_port --protocol=tcp shutdown");
		kill 'INT', $server_runner_pid;
	}
	kill 'INT', $watchdog_pid;
	exit 0;
		
} 
else # watchdog
{ 
	while ( 1 ) {
		sleep 10;
		system( "$opt_mysqladmin -uroot --port=$opt_port --protocol=tcp --silent --connect-timeout=10 ping");
		if ( $? ) {
			print "ERROR: Server got lost!\n";
			kill 'INT', $server_runner_pid if defined $server_runner_pid;
			kill 'INT', $stress_runner_pid;
			exit 1;
		}		
	}

}

sub usage {
	print "The script should be run from mysql-test folder\n";
	print "Usage: perl $0 [options] \n\n";
	print "\t--port=<num> (default $opt_port): \n\t\t server port number\n";
	print "\t--mysqltest=<path> (default $opt_mysqltest): \n\t\t mysqltest binary location\n";
	print "\t--mysqladmin=<path> (default $opt_mysqladmin): \n\t\t mysqladmin binary location\n";
	print "\t--start-server=<1|0> (default $opt_start_server): \n\t\t whether to start server or not\n";
	print "\t--duration=<seconds> (default $opt_duration): \n\t\t test duration\n";
	print "\t--help \n\t\t print this usage and exit\n\n";
	
		
}